package roleypoly

import (
	"log"
	"net/http"
	"strings"
	"time"

	"github.com/goccy/go-json"
	"github.com/gofiber/fiber/v3"
	"github.com/gofiber/fiber/v3/middleware/csrf"
	"github.com/gofiber/fiber/v3/middleware/session"
	"github.com/gofiber/fiber/v3/middleware/static"
	"github.com/gofiber/template/html/v2"

	"git.sapphic.engineer/roleypoly/v4/authmiddleware"
	"git.sapphic.engineer/roleypoly/v4/discord"
	"git.sapphic.engineer/roleypoly/v4/interactions"
	staticfs "git.sapphic.engineer/roleypoly/v4/static"
	"git.sapphic.engineer/roleypoly/v4/templates"
	"git.sapphic.engineer/roleypoly/v4/testing"
)

func CreateFiberApp() *fiber.App {
	viewEngine := html.NewFileSystem(http.FS(templates.FS), ".html")

	app := fiber.New(fiber.Config{
		JSONEncoder: json.Marshal,
		JSONDecoder: json.Unmarshal,
		Views:       viewEngine,
		ViewsLayout: "layouts/main",
	})

	sessionMiddleware, sessionStore := session.NewWithStore()
	sessionStore.RegisterType(authmiddleware.Session{})

	app.Use(sessionMiddleware)
	app.Use(csrf.New(csrf.Config{
		Session: sessionStore,
	}))

	app.Get("/static*", static.New("", static.Config{
		FS:            staticfs.FS,
		Compress:      true,
		CacheDuration: 24 * 8 * time.Hour,
	}))
	app.Get("/favicon.ico", oneStatic)
	app.Get("/manifest.json", oneStatic)
	app.Get("/robots.txt", oneStatic)
	app.Get("/humans.txt", oneStatic)

	return app
}

func oneStatic(c fiber.Ctx) error {
	path := strings.Replace(c.OriginalURL(), "/", "", 1)
	f, err := staticfs.FS.Open(path)
	if err != nil {
		log.Println("oneStatic:", c.OriginalURL(), " failed: ", err)
		return c.SendStatus(500)
	}

	return c.SendStream(f)
}

func SetupControllers(app *fiber.App, dc *discord.DiscordClient, publicKey string, publicBaseURL string) {
	gs := discord.NewGuildService(dc)

	(&testing.TestingController{
		Guilds: gs,
	}).Routes(app.Group("/testing"))

	interactions.NewInteractions(publicKey, publicBaseURL, gs).Routes(app.Group("/interactions"))

	app.Use(authmiddleware.New(dc, []string{}, []string{}))
}

// func getStorageDirectory() string {
// 	path := os.Getenv("STORAGE_DIR")
// 	if path == "" {
// 		path = "./.storage"
// 	}

// 	_, err := os.Stat(path)
// 	if err != nil {
// 		if os.IsNotExist(err) {
// 			err := os.MkdirAll(path, 0744)
// 			if err != nil {
// 				log.Fatalln("failed to create storage directory: ", err)
// 			}
// 		}
// 	}

// 	return path
// }
